from __future__ import annotations

import requests

from osdu_client.exceptions import OSDUAPIError
from osdu_client.services.base import OSDUAPIClient
from osdu_client.utils import urljoin
from osdu_client.validation import validate_data

from .models import (DatasetBulkDeleteBody, DatasetCheckList, DatasetListBody, DatasetLsBody, DatasetPatch,
                     DatasetRegisterBody, ImpersonationTokenRequest, ImpTokenPatchRequest, ImpTokenRequest,
                     RefreshTokenRequest, SubProjectCreateBody, SubProjectPatchBody, TenantCreateBody, UserAddRequest)


class SDMSAPIError(OSDUAPIError):
    pass


class SDMSClient(OSDUAPIClient):
    service_path = "api/seismic-store/v3"

    def get_svcstatus(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "svcstatus")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_svcstatus_access(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "svcstatus/access")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_info(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "info")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def register_new_dataset(
        self,
        *,
        type: str | None = None,
        gtags: list[str] | None = None,
        seismicmeta: dict | None = None,
        openzgy_v1: dict | None = None,
        segy_v1: dict | None = None,
        acls: dict | None = None,
        impersonation_token_context: str | None = None,
        ltag: str | None = None,
        tenantid: str,
        subprojectid: str,
        path: str | None = None,
        datasetid: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context
        if ltag is not None:
            headers["ltag"] = ltag

        params = {}
        if path is not None:
            params["path"] = path

        data = {}
        if type is not None:
            data["type"] = type
        if gtags is not None:
            data["gtags"] = gtags
        if seismicmeta is not None:
            data["seismicmeta"] = seismicmeta
        if openzgy_v1 is not None:
            data["openzgy_v1"] = openzgy_v1
        if segy_v1 is not None:
            data["segy_v1"] = segy_v1
        if acls is not None:
            data["acls"] = acls

        if self.validation:
            validate_data(data, DatasetRegisterBody, SDMSAPIError)

        url = urljoin(
            self.base_url,
            self.service_path,
            "dataset/tenant/%s/subproject/%s/dataset/%s"
            % (tenantid, subprojectid, datasetid),
        )
        response = requests.post(url, headers=headers, params=params, json=data)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_dataset(
        self,
        *,
        impersonation_token_context: str | None = None,
        tenantid: str,
        subprojectid: str,
        path: str | None = None,
        datasetid: str,
        seismicmeta: str | None = None,
        translate_user_info: str | None = None,
        record_version: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {}
        if path is not None:
            params["path"] = path
        if seismicmeta is not None:
            params["seismicmeta"] = seismicmeta
        if translate_user_info is not None:
            params["translate-user-info"] = translate_user_info
        if record_version is not None:
            params["record-version"] = record_version

        url = urljoin(
            self.base_url,
            self.service_path,
            "dataset/tenant/%s/subproject/%s/dataset/%s"
            % (tenantid, subprojectid, datasetid),
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def delete_dataset(
        self,
        *,
        impersonation_token_context: str | None = None,
        tenantid: str,
        subprojectid: str,
        path: str | None = None,
        datasetid: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {}
        if path is not None:
            params["path"] = path

        url = urljoin(
            self.base_url,
            self.service_path,
            "dataset/tenant/%s/subproject/%s/dataset/%s"
            % (tenantid, subprojectid, datasetid),
        )
        response = requests.delete(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def patch_dataset_metadata(
        self,
        *,
        dataset_new_name: str | None = None,
        metadata: dict | None = None,
        filemetadata: dict | None = None,
        last_modified_date: str | None = None,
        gtags: list[str] | None = [],
        ltag: str | None = None,
        readonly: bool | None = None,
        status: str | None = None,
        seismicmeta: dict | None = None,
        openzgy_v1: dict | None = None,
        segy_v1: dict | None = None,
        acls: dict | None = None,
        impersonation_token_context: str | None = None,
        tenantid: str,
        subprojectid: str,
        path: str | None = None,
        datasetid: str,
        close: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {}
        if path is not None:
            params["path"] = path
        if close is not None:
            params["close"] = close

        data = {}
        if dataset_new_name is not None:
            data["dataset_new_name"] = dataset_new_name
        if metadata is not None:
            data["metadata"] = metadata
        if filemetadata is not None:
            data["filemetadata"] = filemetadata
        if last_modified_date is not None:
            data["last_modified_date"] = last_modified_date
        if gtags is not None:
            data["gtags"] = gtags
        if ltag is not None:
            data["ltag"] = ltag
        if readonly is not None:
            data["readonly"] = readonly
        if status is not None:
            data["status"] = status
        if seismicmeta is not None:
            data["seismicmeta"] = seismicmeta
        if openzgy_v1 is not None:
            data["openzgy_v1"] = openzgy_v1
        if segy_v1 is not None:
            data["segy_v1"] = segy_v1
        if acls is not None:
            data["acls"] = acls

        if self.validation:
            validate_data(data, DatasetPatch, SDMSAPIError)

        url = urljoin(
            self.base_url,
            self.service_path,
            "dataset/tenant/%s/subproject/%s/dataset/%s"
            % (tenantid, subprojectid, datasetid),
        )
        response = requests.patch(url, headers=headers, params=params, json=data)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def acquire_lock_for_dataset(
        self,
        *,
        impersonation_token_context: str | None = None,
        tenantid: str,
        subprojectid: str,
        path: str | None = None,
        datasetid: str,
        openmode: str | None = None,
        wid: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {}
        if path is not None:
            params["path"] = path
        if openmode is not None:
            params["openmode"] = openmode
        if wid is not None:
            params["wid"] = wid

        url = urljoin(
            self.base_url,
            self.service_path,
            "dataset/tenant/%s/subproject/%s/dataset/%s/lock"
            % (tenantid, subprojectid, datasetid),
        )
        response = requests.put(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def remove_lock_associated_with_dataset(
        self,
        *,
        impersonation_token_context: str | None = None,
        tenantid: str,
        subprojectid: str,
        path: str | None = None,
        datasetid: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {}
        if path is not None:
            params["path"] = path

        url = urljoin(
            self.base_url,
            self.service_path,
            "dataset/tenant/%s/subproject/%s/dataset/%s/unlock"
            % (tenantid, subprojectid, datasetid),
        )
        response = requests.put(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_dataset_access_permissions(
        self,
        *,
        impersonation_token_context: str | None = None,
        tenantid: str,
        subprojectid: str,
        path: str | None = None,
        datasetid: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {}
        if path is not None:
            params["path"] = path

        url = urljoin(
            self.base_url,
            self.service_path,
            "dataset/tenant/%s/subproject/%s/dataset/%s/permission"
            % (tenantid, subprojectid, datasetid),
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def validate_ctag(
        self,
        *,
        impersonation_token_context: str | None = None,
        tenantid: str,
        subprojectid: str,
        path: str | None = None,
        datasetid: str,
        ctag: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {
            "ctag": ctag,
        }
        if path is not None:
            params["path"] = path

        url = urljoin(
            self.base_url,
            self.service_path,
            "dataset/tenant/%s/subproject/%s/dataset/%s/ctagcheck"
            % (tenantid, subprojectid, datasetid),
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def upsert_tags_to_dataset(
        self,
        *,
        impersonation_token_context: str | None = None,
        tenantid: str,
        subprojectid: str,
        path: str | None = None,
        datasetid: str,
        gtag: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {
            "gtag": gtag,
        }
        if path is not None:
            params["path"] = path

        url = urljoin(
            self.base_url,
            self.service_path,
            "dataset/tenant/%s/subproject/%s/dataset/%s/gtags"
            % (tenantid, subprojectid, datasetid),
        )
        response = requests.put(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def compute_and_get_size_dataset(
        self,
        *,
        impersonation_token_context: str | None = None,
        tenantid: str,
        subprojectid: str,
        datasetid: str,
        path: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {
            "path": path,
        }

        url = urljoin(
            self.base_url,
            self.service_path,
            "dataset/tenant/%s/subproject/%s/dataset/%s/size"
            % (tenantid, subprojectid, datasetid),
        )
        response = requests.post(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_datasets_sizes(
        self,
        *,
        impersonation_token_context: str | None = None,
        tenantid: str,
        subprojectid: str,
        datasetid: str | None = None,
        path: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {}
        if datasetid is not None:
            params["datasetid"] = datasetid
        if path is not None:
            params["path"] = path

        url = urljoin(
            self.base_url,
            self.service_path,
            "dataset/tenant/%s/subproject/%s/size" % (tenantid, subprojectid),
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_content_list(
        self,
        *,
        impersonation_token_context: str | None = None,
        tenantid: str,
        subprojectid: str,
        path: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {}
        if path is not None:
            params["path"] = path

        url = urljoin(
            self.base_url,
            self.service_path,
            "dataset/tenant/%s/subproject/%s/readdsdirfulllist"
            % (tenantid, subprojectid),
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def list_datasets_in_subproject(
        self,
        *,
        type: str | None = None,
        gtags: list[str] | None = None,
        search: str | None = None,
        filter: dict | None = None,
        limit: int | None = None,
        cursor: str | None = None,
        impersonation_token_context: str | None = None,
        tenantid: str,
        subprojectid: str,
        translate_user_info: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {}
        if translate_user_info is not None:
            params["translate-user-info"] = translate_user_info

        data = {}
        if type is not None:
            data["type"] = type
        if gtags is not None:
            data["gtags"] = gtags
        if search is not None:
            data["search"] = search
        if filter is not None:
            data["filter"] = filter
        if limit is not None:
            data["limit"] = limit
        if cursor is not None:
            data["cursor"] = cursor

        if self.validation:
            validate_data(data, DatasetListBody, SDMSAPIError)

        url = urljoin(
            self.base_url,
            self.service_path,
            "dataset/tenant/%s/subproject/%s" % (tenantid, subprojectid),
        )
        response = requests.post(url, headers=headers, params=params, json=data)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def check_datasets_list(
        self,
        *,
        datasets: list[str],
        impersonation_token_context: str | None = None,
        tenantid: str,
        subprojectid: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        data = {
            "datasets": datasets,
        }

        if self.validation:
            validate_data(data, DatasetCheckList, SDMSAPIError)

        url = urljoin(
            self.base_url,
            self.service_path,
            "dataset/tenant/%s/subproject/%s/exist" % (tenantid, subprojectid),
        )
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def list_datasets_sizes(
        self,
        *,
        datasets: list[str],
        impersonation_token_context: str | None = None,
        tenantid: str,
        subprojectid: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        data = {
            "datasets": datasets,
        }

        if self.validation:
            validate_data(data, DatasetCheckList, SDMSAPIError)

        url = urljoin(
            self.base_url,
            self.service_path,
            "dataset/tenant/%s/subproject/%s/sizes" % (tenantid, subprojectid),
        )
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def ls(
        self,
        *,
        impersonation_token_context: str | None = None,
        sdpath: str,
        wmode: str | None = None,
        limit: str | None = None,
        cursor: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {
            "sdpath": sdpath,
        }
        if wmode is not None:
            params["wmode"] = wmode
        if limit is not None:
            params["limit"] = limit
        if cursor is not None:
            params["cursor"] = cursor

        url = urljoin(self.base_url, self.service_path, "utility/ls")
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def ls_post(
        self,
        *,
        sdpath: str,
        wmode: str | None = None,
        limit: int | None = None,
        cursor: str | None = None,
        impersonation_token_context: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        data = {
            "sdpath": sdpath,
        }
        if wmode is not None:
            data["wmode"] = wmode
        if limit is not None:
            data["limit"] = limit
        if cursor is not None:
            data["cursor"] = cursor

        if self.validation:
            validate_data(data, DatasetLsBody, SDMSAPIError)

        url = urljoin(self.base_url, self.service_path, "utility/ls")
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def list_storage_tiers(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "utility/storage-tiers")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def copy_dataset(
        self,
        *,
        impersonation_token_context: str | None = None,
        sdpath_from: str,
        sdpath_to: str,
        lock: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {
            "sdpath_from": sdpath_from,
            "sdpath_to": sdpath_to,
        }
        if lock is not None:
            params["lock"] = lock

        url = urljoin(self.base_url, self.service_path, "utility/cp")
        response = requests.post(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_gcs_access_token(
        self,
        *,
        impersonation_token_context: str | None = None,
        sdpath: str,
        readonly: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {
            "sdpath": sdpath,
        }
        if readonly is not None:
            params["readonly"] = readonly

        url = urljoin(self.base_url, self.service_path, "utility/gcs-access-token")
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_upload_connection_credential_string(
        self,
        *,
        impersonation_token_context: str | None = None,
        sdpath: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {
            "sdpath": sdpath,
        }

        url = urljoin(
            self.base_url, self.service_path, "utility/upload-connection-string"
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_download_connection_credentials_string(
        self,
        *,
        impersonation_token_context: str | None = None,
        sdpath: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if impersonation_token_context is not None:
            headers["impersonation-token-context"] = impersonation_token_context

        params = {
            "sdpath": sdpath,
        }

        url = urljoin(
            self.base_url, self.service_path, "utility/download-connection-string"
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def create_imptoken(
        self,
        *,
        token: str,
        resources: list[dict],
        refresh_url: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {
            "token": token,
            "resources": resources,
            "refresh-url": refresh_url,
        }

        if self.validation:
            validate_data(data, ImpTokenRequest, SDMSAPIError)

        url = urljoin(self.base_url, self.service_path, "imptoken")
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def refresh_imptoken(
        self,
        *,
        token: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {
            "token": token,
        }

        if self.validation:
            validate_data(data, RefreshTokenRequest, SDMSAPIError)

        url = urljoin(self.base_url, self.service_path, "imptoken")
        response = requests.put(url, headers=headers, json=data)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def patch_imptoken(
        self,
        *,
        token: str,
        refresh_url: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {
            "token": token,
            "refresh-url": refresh_url,
        }

        if self.validation:
            validate_data(data, ImpTokenPatchRequest, SDMSAPIError)

        url = urljoin(self.base_url, self.service_path, "imptoken")
        response = requests.patch(url, headers=headers, json=data)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def create_impersonation_token(
        self,
        *,
        resources: list[dict],
        metadata: dict | None = None,
        user_token: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        headers.update(
            {
                "user-token": user_token,
            }
        )

        data = {
            "resources": resources,
        }
        if metadata is not None:
            data["metadata"] = metadata

        if self.validation:
            validate_data(data, ImpersonationTokenRequest, SDMSAPIError)

        url = urljoin(self.base_url, self.service_path, "impersonation-token")
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def refresh_impersonation_token(
        self,
        *,
        impersonation_token: str,
        impersonation_token_context: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        headers.update(
            {
                "impersonation-token": impersonation_token,
                "impersonation-token-context": impersonation_token_context,
            }
        )

        url = urljoin(self.base_url, self.service_path, "impersonation-token")
        response = requests.put(url, headers=headers)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def create_new_subproject(
        self,
        *,
        admin: str | None = None,
        storage_class: str | None = None,
        storage_location: str | None = None,
        access_policy: str | None = "uniform",
        acls: dict | None = None,
        ltag: str | None = None,
        subprojectid: str,
        tenantid: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if ltag is not None:
            headers["ltag"] = ltag

        data = {}
        if admin is not None:
            data["admin"] = admin
        if storage_class is not None:
            data["storage_class"] = storage_class
        if storage_location is not None:
            data["storage_location"] = storage_location
        if access_policy is not None:
            data["access_policy"] = access_policy
        if acls is not None:
            data["acls"] = acls

        if self.validation:
            validate_data(data, SubProjectCreateBody, SDMSAPIError)

        url = urljoin(
            self.base_url,
            self.service_path,
            "subproject/tenant/%s/subproject/%s" % (subprojectid, tenantid),
        )
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_subproject_metadata(
        self,
        *,
        subprojectid: str,
        tenantid: str,
        translate_user_info: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {}
        if translate_user_info is not None:
            params["translate-user-info"] = translate_user_info

        url = urljoin(
            self.base_url,
            self.service_path,
            "subproject/tenant/%s/subproject/%s" % (subprojectid, tenantid),
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def delete_subproject(
        self,
        *,
        subprojectid: str,
        tenantid: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "subproject/tenant/%s/subproject/%s" % (subprojectid, tenantid),
        )
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def patch_subprojects_metadata(
        self,
        *,
        access_policy: str | None = "dataset",
        acls: dict | None = None,
        ltag: str,
        tenantid: str,
        subprojectid: str,
        recursive: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        headers.update(
            {
                "ltag": ltag,
            }
        )

        params = {}
        if recursive is not None:
            params["recursive"] = recursive

        data = {}
        if access_policy is not None:
            data["access_policy"] = access_policy
        if acls is not None:
            data["acls"] = acls

        if self.validation:
            validate_data(data, SubProjectPatchBody, SDMSAPIError)

        url = urljoin(
            self.base_url,
            self.service_path,
            "subproject/tenant/%s/subproject/%s" % (tenantid, subprojectid),
        )
        response = requests.patch(url, headers=headers, params=params, json=data)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def list_subprojects_in_tenant(
        self,
        *,
        tenantid: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "subproject/tenant/%s" % tenantid
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def register_tenant(
        self,
        *,
        gcpid: str,
        esd: str,
        default_acls: str,
        tenantid: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {
            "gcpid": gcpid,
            "esd": esd,
            "default_acls": default_acls,
        }

        if self.validation:
            validate_data(data, TenantCreateBody, SDMSAPIError)

        url = urljoin(self.base_url, self.service_path, "tenant/%s" % tenantid)
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_tenant(
        self,
        *,
        tenantid: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "tenant/%s" % tenantid)
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_tenant_sdpath(
        self,
        *,
        datapartition: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {
            "datapartition": datapartition,
        }

        url = urljoin(self.base_url, self.service_path, "tenant/sdpath")
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def update_user(
        self,
        *,
        email: str,
        path: str,
        group: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {
            "email": email,
            "path": path,
            "group": group,
        }

        if self.validation:
            validate_data(data, UserAddRequest, SDMSAPIError)

        url = urljoin(self.base_url, self.service_path, "user")
        response = requests.put(url, headers=headers, json=data)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_user(
        self,
        *,
        sdpath: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {
            "sdpath": sdpath,
        }

        url = urljoin(self.base_url, self.service_path, "user")
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def delete_user(
        self,
        *,
        email: str,
        path: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {
            "email": email,
            "path": path,
        }

        url = urljoin(self.base_url, self.service_path, "user")
        response = requests.delete(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_user_roles(
        self,
        *,
        sdpath: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {
            "sdpath": sdpath,
        }

        url = urljoin(self.base_url, self.service_path, "user/roles")
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def register_app(
        self,
        *,
        email: str,
        sdpath: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {
            "email": email,
            "sdpath": sdpath,
        }

        url = urljoin(self.base_url, self.service_path, "app")
        response = requests.post(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_app(
        self,
        *,
        sdpath: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {
            "sdpath": sdpath,
        }

        url = urljoin(self.base_url, self.service_path, "app")
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def set_app_trusted(
        self,
        *,
        email: str,
        sdpath: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {
            "email": email,
            "sdpath": sdpath,
        }

        url = urljoin(self.base_url, self.service_path, "app/trusted")
        response = requests.post(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_app_trusted(
        self,
        *,
        sdpath: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {
            "sdpath": sdpath,
        }

        url = urljoin(self.base_url, self.service_path, "app/trusted")
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def delete_all_datasets_in_subproject(
        self,
        *,
        filter: dict | None = None,
        path: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {
            "path": path,
        }

        data = {}
        if filter is not None:
            data["filter"] = filter

        if self.validation:
            validate_data(data, DatasetBulkDeleteBody, SDMSAPIError)

        url = urljoin(self.base_url, self.service_path, "operation/bulk-delete")
        response = requests.put(url, headers=headers, params=params, json=data)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()

    def get_operation_bulk_delete_status(
        self,
        *,
        operation_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "operation/bulk-delete/%s" % operation_id
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise SDMSAPIError(response.text, response.status_code)
        return response.json()



# generated by datamodel-codegen:
#   filename:  swagger.yaml
#   timestamp: 2024-07-12T07:32:17+00:00

from __future__ import annotations

from typing import Any

from pydantic import RootModel


class Field0(RootModel[Any]):
    root: Any



from __future__ import annotations

import requests

from osdu_client.exceptions import OSDUAPIError
from osdu_client.services.base import OSDUAPIClient
from osdu_client.utils import urljoin
from osdu_client.validation import validate_data

from .models import RecordReindexRequest, ReindexRecordsRequest


class IndexerAPIError(OSDUAPIError):
    pass


class IndexerClient(OSDUAPIClient):
    service_path = "/api/indexer/v2"

    def provision_partition(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "partitions/provision")
        response = requests.put(url, headers=headers)
        if not response.ok:
            raise IndexerAPIError(response.text, response.status_code)
        return response.json()

    def reindex_kind(
        self,
        *,
        kind: str,
        cursor: str | None = None,
        force_clean: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {}
        if force_clean is not None:
            params["force_clean"] = force_clean

        data = {
            "kind": kind,
        }
        if cursor is not None:
            data["cursor"] = cursor

        if self.validation:
            validate_data(data, RecordReindexRequest, IndexerAPIError)

        url = urljoin(self.base_url, self.service_path, "reindex")
        response = requests.post(url, headers=headers, params=params, json=data)
        if not response.ok:
            raise IndexerAPIError(response.text, response.status_code)
        return response.json()

    def reindex_partition(
        self,
        *,
        force_clean: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {}
        if force_clean is not None:
            params["force_clean"] = force_clean

        url = urljoin(self.base_url, self.service_path, "reindex")
        response = requests.patch(url, headers=headers, params=params)
        if not response.ok:
            raise IndexerAPIError(response.text, response.status_code)
        return response.json()

    def reindex_records(
        self,
        *,
        record_ids: list[str],
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {
            "recordIds": record_ids,
        }

        if self.validation:
            validate_data(data, ReindexRecordsRequest, IndexerAPIError)

        url = urljoin(self.base_url, self.service_path, "reindex/records")
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise IndexerAPIError(response.text, response.status_code)
        return response.json()

    def get_readiness_check(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "readiness_check")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise IndexerAPIError(response.text, response.status_code)
        return response.json()

    def get_liveness_check(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "liveness_check")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise IndexerAPIError(response.text, response.status_code)
        return response.json()

    def get_info(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "info")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise IndexerAPIError(response.text, response.status_code)
        return response.json()

    def delete_index(
        self,
        *,
        kind: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {
            "kind": kind,
        }

        url = urljoin(self.base_url, self.service_path, "index")
        response = requests.delete(url, headers=headers, params=params)
        if not response.ok:
            raise IndexerAPIError(response.text, response.status_code)
        return response.json()



# generated by datamodel-codegen:
#   filename:  swagger.yaml
#   timestamp: 2024-07-12T07:32:18+00:00

from __future__ import annotations

from datetime import date
from enum import Enum
from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel, Field, RootModel, constr


class Acl(BaseModel):
    viewers: Optional[List[str]] = None
    owners: Optional[List[str]] = None


class Status(Enum):
    compliant = "compliant"
    incompliant = "incompliant"


class Legal(BaseModel):
    legaltags: Optional[List[str]] = Field(None)
    otherRelevantDataCountries: List[str] = Field(...)
    status: Optional[Status] = None


class RecordAncestry(BaseModel):
    parents: Optional[List[str]] = Field(None)


class AppError(BaseModel):
    code: Optional[int] = None
    reason: Optional[str] = None
    message: Optional[str] = None


class CreateUpdateRecordsResponse(BaseModel):
    recordCount: Optional[int] = None
    recordIds: Optional[List[str]] = None
    skippedRecordIds: Optional[List[str]] = None
    recordIdVersions: Optional[List[str]] = None


class RecordVersionModel(BaseModel):
    id: Optional[str] = None
    version: Optional[str] = None


class StackTraceItem(BaseModel):
    classLoaderName: Optional[str] = None
    moduleName: Optional[str] = None
    moduleVersion: Optional[str] = None
    methodName: Optional[str] = None
    fileName: Optional[str] = None
    lineNumber: Optional[int] = None
    nativeMethod: Optional[bool] = None
    className: Optional[str] = None


class Cause(BaseModel):
    stackTrace: Optional[List[StackTraceItem]] = None
    message: Optional[str] = None
    localizedMessage: Optional[str] = None


class SuppressedItem(BaseModel):
    stackTrace: Optional[List[StackTraceItem]] = None
    message: Optional[str] = None
    localizedMessage: Optional[str] = None


class PairStringString(BaseModel):
    value: Optional[str] = None
    key: Optional[str] = None
    left: Optional[str] = None
    right: Optional[str] = None


class MultiRecordIds(BaseModel):
    records: List[str] = Field(..., max_items=100, min_items=0)
    attributes: Optional[List[str]] = None


class MultiRecordRequest(BaseModel):
    records: List[str] = Field(..., max_items=20, min_items=1)


class ConversionStatus(BaseModel):
    id: Optional[str] = None
    status: Optional[str] = None
    errors: Optional[List[str]] = None


class MultiRecordResponse(BaseModel):
    records: Optional[List[str]] = None
    notFound: Optional[List[str]] = None
    conversionStatuses: Optional[List[ConversionStatus]] = None


class PatchOperation(BaseModel):
    op: Optional[str] = None
    path: Optional[str] = None
    value: List[str]


class RecordQuery(BaseModel):
    ids: List[str] = Field(..., max_items=500, min_items=1)


class BulkUpdateRecordsResponse(BaseModel):
    recordCount: Optional[int] = None
    recordIds: Optional[List[str]] = None
    notFoundRecordIds: Optional[List[str]] = None
    unAuthorizedRecordIds: Optional[List[str]] = None
    lockedRecordIds: Optional[List[str]] = None


class JsonPatch(BaseModel):
    pass


class RecordQueryPatch(BaseModel):
    ids: Optional[List[str]] = Field(
        None,
        description="List of record ids. Each record id must follow the naming convention {Data-Partition-Id}:{dataset-name}:{record-type}:{version}.",
    )


class PatchRecordsResponse(BaseModel):
    recordCount: Optional[int] = None
    recordIds: Optional[List[str]] = None
    notFoundRecordIds: Optional[List[str]] = None
    failedRecordIds: Optional[List[str]] = None
    errors: Optional[List[str]] = None


class RecordVersions(BaseModel):
    recordId: Optional[str] = None
    versions: Optional[List[int]] = None


class DatastoreQueryResult(BaseModel):
    cursor: Optional[str] = None
    results: Optional[List[str]] = None


class ConnectedOuterService(BaseModel):
    name: Optional[str] = None
    version: Optional[str] = None


class VersionInfo(BaseModel):
    groupId: Optional[str] = None
    artifactId: Optional[str] = None
    version: Optional[str] = None
    buildTime: Optional[str] = None
    branch: Optional[str] = None
    commitId: Optional[str] = None
    commitMessage: Optional[str] = None
    connectedOuterServices: Optional[List[ConnectedOuterService]] = None


class Status1(BaseModel):
    kind: Optional[str] = None
    totalRecords: Optional[int] = None
    processedRecords: Optional[int] = None
    state: Optional[str] = None
    startedAt: Optional[date] = None
    elapsedTime: Optional[str] = None
    avgProcessingTimePerMinute: Optional[int] = None


class ReplayStatus(BaseModel):
    kind: Optional[str] = None
    replayId: str
    totalRecords: int
    processedRecords: int
    overallState: str
    startedAt: Optional[date] = None
    filter: Optional[List[Dict[str, Any]]] = None
    status: Union[List[Dict[str, Any]], Status1]


class Status2(BaseModel):
    replayId: int = Field(..., example=1000)


class Kind(RootModel[List[str]]):
    root: List[str]


class Record(BaseModel):
    id: Optional[constr(pattern=r"^[\w\-\.]+:[\w\-\.]+:[\w\-\.\:\%]+$")] = None
    version: Optional[int] = None
    kind: Optional[str] = None
    acl: Acl
    legal: Optional[Legal] = None
    data: Dict[str, Dict[str, Any]]
    ancestry: Optional[RecordAncestry] = None
    meta: Optional[List[Dict[str, Dict[str, Any]]]] = None
    tags: Optional[Dict[str, str]] = None
    createUser: Optional[str] = None
    createTime: Optional[str] = None
    modifyUser: Optional[str] = None
    modifyTime: Optional[str] = None


class CopyRecordReferencesModel(BaseModel):
    target: Optional[str] = None
    records: Optional[List[RecordVersionModel]] = None


class DeleteRecordsException(BaseModel):
    cause: Optional[Cause] = None
    stackTrace: Optional[List[StackTraceItem]] = None
    notDeletedRecords: Optional[List[PairStringString]] = None
    message: Optional[str] = None
    suppressed: Optional[List[SuppressedItem]] = None
    localizedMessage: Optional[str] = None


class MultiRecordInfo(BaseModel):
    records: Optional[List[Record]] = None
    invalidRecords: Optional[List[str]] = None
    retryRecords: Optional[List[str]] = None


class RecordBulkUpdateParam(BaseModel):
    query: RecordQuery
    ops: List[PatchOperation]


class PatchRecordsRequestModel(BaseModel):
    query: RecordQueryPatch
    ops: JsonPatch


class ReplayRequest(BaseModel):
    operation: str
    filter: Optional[Kind] = None



from __future__ import annotations

import requests

from osdu_client.exceptions import OSDUAPIError
from osdu_client.services.base import OSDUAPIClient
from osdu_client.utils import urljoin


class WellDeliveryAPIError(OSDUAPIError):
    pass


class WellDeliveryClient(OSDUAPIClient):
    service_path = "/api/well-delivery"

    def create_or_update_storage(
        self,
        *,
        entity: dict,
        type: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {
            "Entity": entity,
        }

        url = urljoin(self.base_url, self.service_path, "storage/v1/%s" % type)
        response = requests.put(url, headers=headers, json=data)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_entity(
        self,
        *,
        type: str,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "storage/v1/%s/%s" % (type, id))
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def delete_entity(
        self,
        *,
        type: str,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "storage/v1/%s/%s" % (type, id))
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_entity_version(
        self,
        *,
        type: str,
        id: str,
        version: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "storage/v1/%s/%s/%s" % (type, id, version),
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def delete_storage_version(
        self,
        *,
        type: str,
        id: str,
        version: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "storage/v1/%s/%s/%s" % (type, id, version),
        )
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_storage_version(
        self,
        *,
        type: str,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "storage/v1/%s/versions/%s" % (type, id)
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def purge_entity(
        self,
        *,
        type: str,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "storage/v1/%s/%s:purge" % (type, id)
        )
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_version_number_list_well_activity_program(
        self,
        *,
        well_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "wellActivityPrograms/v1/versions/by_well/%s" % well_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_well_activity_program(
        self,
        *,
        well_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "wellActivityPrograms/v1/by_well/%s" % well_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_well_activity_program_version(
        self,
        *,
        well_id: str,
        wap_version: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "wellActivityPrograms/v1/by_well/%s/%s" % (well_id, wap_version),
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_well_activity_program_with_refs(
        self,
        *,
        well_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "wellActivityPrograms/v1/reference_tree/by_well/%s" % well_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_well_activity_program_version_with_refs(
        self,
        *,
        well_id: str,
        wap_version: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "wellActivityPrograms/v1/reference_tree/by_well/%s/%s"
            % (well_id, wap_version),
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_well_activity_programs_full_content_by_well(
        self,
        *,
        well_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "wellActivityPrograms/v1/full_content/by_well/%s" % well_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_activity_plans_by_well(
        self,
        *,
        well_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "activityPlans/v1/by_well/%s" % well_id
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_version_number_list_actual_well(
        self,
        *,
        name: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "wells/v1/versions/by_name/%s:actual" % name,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_well_by_name_actual(
        self,
        *,
        name: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "wells/v1/by_name/%s:actual" % name
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_well_version_by_name_actual(
        self,
        *,
        name: str,
        version: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "wells/v1/by_name/%s/%s:actual" % (name, version),
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_version_number_list_planned_well(
        self,
        *,
        name: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "wells/v1/versions/by_name/%s:planned" % name,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_well_by_name_planned(
        self,
        *,
        name: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "wells/v1/by_name/%s:planned" % name
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_well_version_by_name_planned(
        self,
        *,
        name: str,
        version: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "wells/v1/by_name/%s/%s:planned" % (name, version),
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_version_number_list_actual_wellbore(
        self,
        *,
        well_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "wellbores/v1/versions/by_well/%s:actual" % well_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_wellbore_actual(
        self,
        *,
        well_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "wellbores/v1/by_well/%s:actual" % well_id
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_wellbore_version_actual(
        self,
        *,
        well_id: str,
        wellbore_version: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "wellbores/v1/by_well/%s/%s:actual" % (well_id, wellbore_version),
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_version_number_list_planned_wellbore(
        self,
        *,
        well_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "wellbores/v1/versions/by_well/%s:planned" % well_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_wellbore_planned(
        self,
        *,
        well_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "wellbores/v1/by_well/%s:planned" % well_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_wellbore_version_planned(
        self,
        *,
        well_id: str,
        wellbore_version: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "wellbores/v1/by_well/%s/%s:planned" % (well_id, wellbore_version),
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_hole_sections_v1_by_wellbore(
        self,
        *,
        wellbore_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "holeSections/v1/by_wellbore/%s" % wellbore_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_bha_runs_v1_by_hole_section(
        self,
        *,
        hole_section_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "bhaRuns/v1/by_holeSection/%s" % hole_section_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_bha_runs_v1_by_wellbore_actual(
        self,
        *,
        wellbore_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "bhaRuns/v1/by_wellbore/%s:actual" % wellbore_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def query_v1_by_well_batch(
        self,
        *,
        well_ids: list[str],
        content__type: str,
        type: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        headers.update(
            {
                "Content-Type": content__type,
            }
        )

        data = {
            "well_ids": well_ids,
        }

        url = urljoin(
            self.base_url, self.service_path, "query/v1/by_well/%s:batch" % type
        )
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def query_v1_by_wellbore_planned(
        self,
        *,
        type: str,
        wellbore_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "query/v1/%s/by_wellbore/%s:planned" % (type, wellbore_id),
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def query_v1_by_wellbore_actual(
        self,
        *,
        type: str,
        wellbore_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "query/v1/%s/by_wellbore/%s:actual" % (type, wellbore_id),
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_wellbore_fluids_programs(
        self,
        *,
        wellbore_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "fluidsPrograms/v1/by_wellbore/%s" % wellbore_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_wellbore_operations_reports(
        self,
        *,
        wellbore_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "operationsReports/v1/by_wellbore/%s" % wellbore_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_wellbore_operations_reports_latest(
        self,
        *,
        wellbore_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "operationsReports/v1/latest/by_wellbore/%s" % wellbore_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_operations_reports_by_time_range(
        self,
        *,
        start_time: str,
        end_time: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "operationsReports/v1/by_timeRange/%s/%s" % (start_time, end_time),
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_operations_reports_with_refs_by_operations_report_id(
        self,
        *,
        operations_report_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "operationsReports/v1/reference_tree/by_operationsReport/%s"
            % operations_report_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()

    def get_wellbore_fluids_reports(
        self,
        *,
        wellbore_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "fluidsReports/v1/by_wellbore/%s" % wellbore_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellDeliveryAPIError(response.text, response.status_code)
        return response.json()



# generated by datamodel-codegen:
#   filename:  swagger.yaml
#   timestamp: 2024-07-12T07:32:21+00:00

from __future__ import annotations

from datetime import datetime
from enum import Enum
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict, Field, RootModel, constr


class Location(BaseModel):
    SignedURL: Optional[str] = Field(
        None, json_schema_extra=dict(example="GCS signed URL")
    )
    FileSource: Optional[str] = Field(
        None,
        description="Relative location of file",
        json_schema_extra=dict(example="AXB564XVCY\\SHVCU7632"),
    )


class EndianEnum(str, Enum):
    BIG = "BIG"
    LITTLE = "LITTLE"


class FileSourceInfo(BaseModel):
    Name: Optional[str] = Field(None, description="user-friendly file name.")
    PreloadFilePath: Optional[str] = Field(
        None,
        description="File system path to the data file as it existed before loading to the data platform",
    )
    FileSource: str = Field(
        ..., description="Relative file path for the data in the file"
    )
    PreloadFileCreateUser: Optional[str] = Field(
        None,
        description="Optional user name or reference, who created the file prior to up-loading to the platform.",
    )
    PreloadFileCreateDate: Optional[str] = Field(
        None,
        description="Optional create date and time of the file prior to uploading to the platform.",
    )
    PreloadFileModifyUser: Optional[str] = Field(
        None,
        description="Optional user name or reference, who last modified the file prior to up-loading to the platform.",
    )
    PreloadFileModifyDate: Optional[str] = Field(
        None,
        description="Optional last modified date and time of the file prior to up-loading to the platform.",
    )
    FileSize: Optional[str] = Field(
        None,
        description="Length of file in bytes. Implemented as string. The value must be convertible to a long integer (sizes can become very large).",
    )
    EncodingFormatTypeID: Optional[
        constr(
            pattern=r"^srn:<namespace>:reference-data\\/EncodingFormatType:[^:]+:[0-9]*$"
        )
    ] = Field(None, description="Encoding Format Type ID")

    model_config = ConfigDict(use_enum_values=True)


class DatasetProperties(BaseModel):
    FileSourceInfo: FileSourceInfo


class ExtensionProperties(BaseModel):
    kind: Optional[str] = Field(
        None,
        description="The schema ID for this schema fragment",
        json_schema_extra=dict(example="os:npd:csvFileExtDetails:1.0.0"),
        title="Extension Schema ID",
    )


class DownloadResponse(BaseModel):
    SignedURL: Optional[str]


class FileMetadataResponse(BaseModel):
    Id: Optional[str]


class DeliveryGetFileSignedURLRequest(BaseModel):
    srn: Optional[List[str]] = Field(None, description="A list of SRNs to fetch")


class Processed(BaseModel):
    signedUrl: Optional[str]


class DeliveryGetFileSignedURLResponse(BaseModel):
    unprocessed: Optional[List[str]] = Field(
        None, description="A list of SRNs which could not be processed"
    )
    processed: Optional[Dict[str, Processed]] = Field(
        None, description="Each key is equal to an SRN that was able to be processed"
    )


class FileID(RootModel[Any]):
    root: constr(pattern=r"^[\w,\s-]+(\.\w+)?$")


class DateTime(RootModel[datetime]):
    root: datetime


class Driver(Enum):
    GCP = "GCP"
    AWS = "AWS"


class FileLocation(BaseModel):
    FileID: Optional[FileID]
    Driver: Optional[Driver]
    Location: Optional[str]
    CreatedAt: Optional[datetime]
    CreatedBy: Optional[str]


class Acl(BaseModel):
    viewers: List[str] = Field(
        ...,
        description="List of valid groups which will have view/read privileges over the record.",
    )
    owners: List[str] = Field(
        ...,
        description="List of valid groups which will have write privileges over the record.",
    )


class Legal(BaseModel):
    legaltags: Optional[List[str]] = Field(
        None,
        description="The list of legal tags, see compliance API.",
        title="Legal Tags",
    )
    otherRelevantDataCountries: Optional[List[str]] = Field(
        None,
        description="The list of other relevant data countries using the ISO 2-letter codes, see compliance API.",
        title="Other Relevant Data Countries",
    )
    status: Optional[str] = Field(
        None, description="The legal status.", title="Legal Status"
    )


class FileSourceInfoObject(BaseModel):
    Name: Optional[str] = Field(None, description="user-friendly file name.")
    PreloadFilePath: Optional[str] = Field(
        None,
        description="File system path to the data file as it existed before loading to the data platform",
    )
    FileSource: Optional[str] = Field(
        None, description="Relative file path for the data in the file"
    )
    PreloadFileCreateUser: Optional[str] = Field(
        None,
        description="Optional user name or reference, who created the file prior to up-loading to the platform.",
    )
    PreloadFileCreateDate: Optional[str] = Field(
        None,
        description="Optional create date and time of the file prior to uploading to the platform.",
    )
    PreloadFileModifyUser: Optional[str] = Field(
        None,
        description="Optional user name or reference, who last modified the file prior to up-loading to the platform.",
    )
    PreloadFileModifyDate: Optional[str] = Field(
        None,
        description="Optional last modified date and time of the file prior to up-loading to the platform.",
    )
    FileSize: Optional[str] = Field(
        None,
        description="Length of file in bytes. Implemented as string. The value must be convertible to a long integer (sizes can become very large).",
    )
    EncodingFormatTypeID: Optional[
        constr(
            pattern=r"^srn:<namespace>:reference-data\\/EncodingFormatType:[^:]+:[0-9]*$"
        )
    ] = Field(None, description="Encoding Format Type ID")
    Checksum: Optional[constr(pattern=r"^([0-9a-fA-F]{2})+$")] = Field(
        None,
        description="Checksum of file bytes - a hexadecimal number with even number of bytes.",
    )
    ChecksumAlgorithm: Optional[str] = Field(
        None, description="The name of the checksum algorithm e.g. MD5, SHA-256."
    )


class Kind(Enum):
    CRS = "CRS"
    Unit = "Unit"
    Measurement = "Measurement"
    AzimuthReference = "AzimuthReference"
    DateTime = "DateTime"


class MetaItem(BaseModel):
    kind: Kind = Field(
        ...,
        description="The kind of reference, unit, measurement, CRS or azimuth reference.",
        title="Reference Kind",
    )
    name: Optional[str] = Field(
        None,
        description="The name of the CRS or the symbol/name of the unit",
        json_schema_extra=dict(
            example='["NAD27 * OGP-Usa Conus / North Dakota South [32021,15851]","ft"]'
        ),
        title="Name or Symbol",
    )
    persistableReference: str = Field(
        ...,
        description="The persistable reference string uniquely identifying the CRS or Unit",
        json_schema_extra=dict(
            example='{"scaleOffset":{"scale":0.3048006096012192,"offset":0.0},"symbol":"ftUS","baseMeasurement":{"ancestry":"Length","type":"UM"},"type":"USO"}'
        ),
        title="Persistable Reference",
    )
    propertyNames: Optional[List[str]] = Field(
        None,
        description='The list of property names, to which this meta data item provides Unit/CRS context to. Data structures, which come in a single frame of reference, can register the property name, others require a full path like "data.structureA.propertyB" to define a unique context.',
        json_schema_extra=dict(
            example=["elevationFromMsl", '"totalDepthMdDriller', "wellHeadProjected"]
        ),
        title="Attribute Names",
    )
    propertyValues: Optional[List[str]] = Field(
        None,
        description="The list of property values, to which this meta data item provides Unit/CRS context to. Typically a unit symbol is a value to a data structure; this symbol is then registered in this propertyValues array and the persistableReference provides the absolute reference.",
        json_schema_extra=dict(example=["F", "ftUS", "deg"]),
        title="Attribute Names",
    )
    uncertainty: Optional[float] = Field(
        None,
        description="The uncertainty of the values measured given the unit or CRS unit.",
        title="Uncertainty",
    )


class ToManyRelationship(BaseModel):
    confidences: Optional[List[float]] = Field(
        None,
        description="The confidences of the relationships. Keep all the arrays ordered and aligned.",
        title="Relationship Confidences",
    )
    ids: Optional[List[str]] = Field(
        None,
        description="The ids of the related objects. It is populated for an explicit relationship where the target entity is present as a record in the data ecosystem. Keep all the arrays ordered and aligned.",
        title="Related Object Id",
    )
    names: Optional[List[str]] = Field(
        None,
        description="The names or natural keys of the related objects. Keep all the arrays ordered and aligned.",
        title="Related Object Names",
    )
    versions: Optional[List[float]] = Field(
        None,
        description="The specific version numbers of the related instances. This is only specified if a specific version is required. If not populated the last version is implied. Keep all the arrays ordered and aligned.",
        title="To Many Relationship",
    )


class ToOneRelationship(BaseModel):
    confidence: Optional[float] = Field(
        None,
        description="The confidence of the relationship. If the property is absent a well-known relation is implied.",
        title="Relationship Confidence",
        json_schema_extra=dict(
            example=1,
        ),
    )
    id: Optional[str] = Field(
        None,
        description="The id of the related object in the Data Ecosystem. If set, the id has priority over the natural key in the name property.",
        title="Related Object Id",
        json_schema_extra=dict(
            example="data_partition:namespace:entity_845934c40e8d922bc57b678990d55722",
        ),
    )
    name: Optional[str] = Field(
        None,
        description="The name or natural key of the related object. This property is required if the target object id could not (yet) be identified.",
        title="Related Object Name",
        json_schema_extra=dict(
            example="Survey ST2016",
        ),
    )
    version: Optional[float] = Field(
        None,
        description="The version number of the related entity. If no version number is specified, the last version is implied.",
        title="Entity Version Number",
    )


class LinkList(RootModel[Optional[Dict[str, List[str]]]]):
    root: Optional[Dict[str, List[str]]] = Field(
        None,
        description="An array of one or more entity references in the data lake.",
        title="Link List",
    )


class Error(BaseModel):
    message: Optional[str]
    reason: Optional[str]
    domain: Optional[str]


class ConnectedOuterService(BaseModel):
    name: Optional[str] = Field(None, description="Connected outer service name.")
    version: Optional[str] = Field(None, description="Connected outer service version.")


class StorageLocation(BaseModel):
    CSP_SPECIFIC_PROPERTY_1: Optional[str]
    CSP_SPECIFIC_PROPERTY_2: Optional[str]
    CSP_SPECIFIC_PROPERTY_3: Optional[str]


class StorageInstructionsResponse(BaseModel):
    providerKey: Optional[str] = Field(None, json_schema_extra=dict(example="AZURE"))
    storageLocation: Optional[StorageLocation]


class RetrievalProperties(BaseModel):
    CSP_SPECIFIC_PROPERTY_1: Optional[str]
    CSP_SPECIFIC_PROPERTY_2: Optional[str]
    CSP_SPECIFIC_PROPERTY_3: Optional[str]


class Dataset(BaseModel):
    datasetRegistryId: Optional[str] = Field(
        None,
        json_schema_extra=dict(
            example="opendes:dataset--FileCollection.Generic:8118591ee2a24eada7152e54b369e99a",
        ),
    )
    retrievalProperties: Optional[RetrievalProperties]
    providerKey: Optional[str] = Field(None, json_schema_extra=dict(example="AZURE"))


class RetrievalInstructionsResponse(BaseModel):
    datasets: Optional[List[Dataset]]


class CopyDmsResponseItem(BaseModel):
    success: Optional[bool]
    datasetBlobStoragePath: Optional[str]


class CopyDmsResponse(RootModel[List[CopyDmsResponseItem]]):
    root: List[CopyDmsResponseItem]


class LocationRequest(BaseModel):
    FileID: Optional[FileID]


class LocationResponse(BaseModel):
    FileID: Optional[FileID]
    Location: Optional[Dict[str, str]] = Field(
        None, json_schema_extra=dict(example={"SignedURL": "GCS signed URL"})
    )


class SourceLocationResponse(BaseModel):
    FileID: Optional[FileID]
    Location: Optional[Location]


class DatasetProperties1(BaseModel):
    FileCollectionPath: str = Field(
        ...,
        description="The mandatory path to the file collection. A FileCollectionPath should represent folder level access to a set of files.",
    )
    IndexFilePath: Optional[str] = Field(
        None, description="An optional path to an index file."
    )
    FileSourceInfo: Optional[List[FileSourceInfoObject]]


class FileCollections(BaseModel):
    Name: str = Field(
        ...,
        description="An optional name of the dataset, e.g. a user friendly file or file collection name.",
    )
    Description: Optional[str] = Field(
        None, description="An optional, textual description of the dataset."
    )
    TotalSize: Optional[constr(pattern=r"^[0-9]+$")] = Field(
        None,
        description="Total size of the dataset in bytes; for files it is the same as declared in FileSourceInfo.FileSize or the sum of all individual files. Implemented as string. The value must be convertible to a long integer (sizes can become very large).",
    )
    EncodingFormatTypeID: Optional[
        constr(
            pattern=r"^srn:<namespace>:reference-data\\/EncodingFormatType:[^:]+:[0-9]*$"
        )
    ] = Field(None, description="Encoding Format Type ID")
    SchemaFormatTypeID: Optional[
        constr(
            pattern=r"^srn:<namespace>:reference-data\\/SchemaFormatType:[^:]+:[0-9]*$"
        )
    ] = Field(None, description="Schema Format Type ID")
    Endian: EndianEnum = Field(
        ...,
        description="Endianness of binary value. Enumeration- \\BIG\\ \\LITTLE\\.  If absent applications will need to interpret from context indicators.",
    )
    Checksum: Optional[constr(pattern=r"^[0-9a-fA-F]32}$")] = Field(
        None, description="MD5 checksum of file bytes - a 32 byte hexadecimal number"
    )
    DatasetProperties: DatasetProperties1

    model_config = ConfigDict(use_enum_values=True)


class FileDetails(BaseModel):
    TargetKind: Optional[str] = Field(
        None,
        description="The target kind or schema ID which is to be used by the parser.",
        json_schema_extra=dict(
            example="os:npd:wellbore:1:*.*",
        ),
        title="Target Schema ID",
    )
    FileType: Optional[str] = Field(
        None,
        description="Type of File to decide what kind of ingestion to be triggered",
        json_schema_extra=dict(example="csv"),
        title="File Type",
    )
    FrameOfReference: Optional[List[MetaItem]] = Field(
        None,
        description="The list metaItem definitions which maps a named frame of reference symbol or name to the self-contained persistableReference.",
        title="Frame of Reference for data present in file.",
    )
    ExtensionProperties: Optional[ExtensionProperties]
    ParentReference: Optional[str] = Field(
        None,
        description="The parent reference for this file.",
        json_schema_extra=dict(example="CSBE0417"),
        title="Parent Reference",
    )


class FileLocationRequest(BaseModel):
    FileID: Optional[FileID]


class FileLocationResponse(BaseModel):
    Driver: Optional[Driver]
    Location: Optional[str]


class FileListRequest(BaseModel):
    TimeFrom: Optional[DateTime]
    TimeTo: Optional[DateTime]
    PageNum: Optional[int]
    Items: Optional[int]
    UserID: Optional[str]


class FileListResponse(BaseModel):
    content: Optional[List[FileLocation]]
    number: Optional[int]
    numberOfElements: Optional[int]
    size: Optional[int]


class FileCollectionRecord(BaseModel):
    id: Optional[str] = Field(
        None,
        description="Unique identifier generated by the system for the directory metadata record.",
    )
    kind: str = Field(
        ...,
        description="Kind of data being ingested. Must follow the naming convention:data-Partition-Id}:dataset-name}:record-type}:version}.",
        json_schema_extra=dict(
            example="osdu:wks:dataset--FileCollection.Generic:1.0.0"
        ),
    )
    acl: Acl
    legal: Legal
    data: FileCollections
    ancestry: Optional[LinkList]


class Relationships(BaseModel):
    parentEntity: Optional[ToOneRelationship]
    relatedItems: Optional[ToManyRelationship]


class ErrorModel(BaseModel):
    errors: Optional[List[Error]]
    code: Optional[int]
    message: Optional[str]


class VersionInfo(BaseModel):
    groupId: Optional[str] = Field(None, description="Maven artifact group ID.")
    actifactId: Optional[str] = Field(None, description="Maven artifact ID.")
    version: Optional[str] = Field(None, description="Maven artifact version")
    buildTime: Optional[str] = Field(None, description="Maven artifact build time")
    branch: Optional[str] = Field(None, description="Current git branch")
    commitId: Optional[str] = Field(None, description="Latest commit hash")
    commitMessage: Optional[str] = Field(None, description="Latest commit message")
    connectedOuterServices: Optional[List[ConnectedOuterService]] = Field(
        None, description="Connected outer services information"
    )


class ExtensionPropertiesModel(BaseModel):
    Name: Optional[str] = Field(
        None,
        description="The name of the file. Note- this is an additional property, which is not part of OSDU File.1.0.0",
        json_schema_extra=dict(example="File"),
        title="File Name",
    )
    Classification: Optional[str] = Field(
        None,
        description="The well-known entity classification code.",
        json_schema_extra=dict(example="Raw File"),
        title="File Classification",
    )
    Description: Optional[str] = Field(
        None,
        description="A text describing the entity.",
        json_schema_extra=dict(example="An text further describing this file example."),
        title="Entity Description",
    )
    ExternalIds: Optional[List[str]] = Field(
        None,
        description="An array of identities (e.g. some kind if URL to be resolved in an external data store) which links to external realizations of the same entity.",
        title="Array of External IDs",
    )
    FileDateCreated: Optional[datetime] = Field(
        None,
        description="The UTC date time of the file creation",
        json_schema_extra=dict(example="2013-03-22T11:16:03Z"),
        title="Creation Date and Time",
    )
    FileDateModified: Optional[datetime] = Field(
        None,
        description="The UTC date time of the last file modification",
        json_schema_extra=dict(example="2013-03-22T11:16:03Z"),
        title="Last Modification Date and Time",
    )
    FileContentsDetails: Optional[FileDetails]
    relationships: Optional[Relationships]


class Files(BaseModel):
    Name: str = Field(
        ...,
        description="An optional name of the dataset, e.g. a user friendly file or file collection name.",
    )
    Description: Optional[str] = Field(
        None, description="An optional, textual description of the dataset."
    )
    TotalSize: Optional[constr(pattern=r"^[0-9]+$")] = Field(
        None,
        description="Total size of the dataset in bytes; for files it is the same as declared in FileSourceInfo.FileSize or the sum of all individual files. Implemented as string. The value must be convertible to a long integer (sizes can become very large).",
    )
    EncodingFormatTypeID: Optional[
        constr(
            pattern=r"^srn:<namespace>:reference-data\\/EncodingFormatType:[^:]+:[0-9]*$"
        )
    ] = Field(None, description="Encoding Format Type ID")
    SchemaFormatTypeID: Optional[
        constr(
            pattern=r"^srn:<namespace>:reference-data\\/SchemaFormatType:[^:]+:[0-9]*$"
        )
    ] = Field(None, description="Schema Format Type ID")
    Endian: EndianEnum = Field(
        ...,
        description="Endianness of binary value. Enumeration- \\BIG\\ \\LITTLE\\.  If absent applications will need to interpret from context indicators.",
    )
    Checksum: Optional[constr(pattern=r"^[0-9a-fA-F]32}$")] = Field(
        None, description="MD5 checksum of file bytes - a 32 byte hexadecimal number"
    )
    DatasetProperties: DatasetProperties
    ExtensionProperties: Optional[ExtensionPropertiesModel] = Field(
        None, title="File DMS Extension Properties"
    )


class Record(BaseModel):
    id: Optional[str] = Field(
        None,
        description="Unique identifier generated by the system for the file metadata record.",
    )
    kind: str = Field(
        ...,
        description="Kind of data being ingested. Must follow the naming convention:data-Partition-Id}:dataset-name}:record-type}:version}.",
        json_schema_extra=dict(example="osdu:wks:dataset--File.Generic:1.0.0"),
    )
    acl: Acl
    legal: Legal
    data: Files
    ancestry: Optional[LinkList]


class RecordVersion(BaseModel):
    id: Optional[str] = Field(
        None,
        description="Unique identifier generated by the system for the file metadata record.",
    )
    kind: Optional[str] = Field(
        None,
        description="Kind of data being ingested. Must follow the naming convention:data-Partition-Id}:dataset-name}:record-type}:version}.",
        json_schema_extra=dict(example="osdu:wks:dataset--File.Generic:1.0.0"),
    )
    acl: Optional[Acl]
    legal: Optional[Legal]
    data: Optional[Files]
    ancestry: Optional[LinkList]
    version: Optional[int] = Field(
        None,
        description="The version number of this OSDU resource; set by the framework.",
        json_schema_extra=dict(example=1831253916104085),
        title="Version Number",
    )


class ApplicationError(BaseModel):
    error: Optional[ErrorModel]



from __future__ import annotations

import requests

from osdu_client.exceptions import OSDUAPIError
from osdu_client.services.base import OSDUAPIClient
from osdu_client.utils import urljoin
from osdu_client.validation import validate_data

from .models import Secret


class SecretAPIError(OSDUAPIError):
    pass


class SecretClient(OSDUAPIClient):
    service_path = "/api/secret/v1"

    def get_secret(
        self,
        *,
        secret_name: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "secrets/%s" % secret_name)
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise SecretAPIError(response.text, response.status_code)
        return response.json()

    def update_secret(
        self,
        *,
        id: str | None = None,
        key: str | None = None,
        value: str | None = None,
        created_at: str | None = None,
        enabled: bool | None = None,
        secret_name: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {}
        if id is not None:
            data["id"] = id
        if key is not None:
            data["key"] = key
        if value is not None:
            data["value"] = value
        if created_at is not None:
            data["createdAt"] = created_at
        if enabled is not None:
            data["enabled"] = enabled

        if self.validation:
            validate_data(data, Secret, SecretAPIError)

        url = urljoin(self.base_url, self.service_path, "secrets/%s" % secret_name)
        response = requests.put(url, headers=headers, json=data)
        if not response.ok:
            raise SecretAPIError(response.text, response.status_code)
        return response.json()

    def delete_secret(
        self,
        *,
        secret_name: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "secrets/%s" % secret_name)
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise SecretAPIError(response.text, response.status_code)
        return response.json()

    def list_secrets(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "secrets")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise SecretAPIError(response.text, response.status_code)
        return response.json()

    def create_secrets(
        self,
        *,
        id: str | None = None,
        key: str | None = None,
        value: str | None = None,
        created_at: str | None = None,
        enabled: bool | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {}
        if id is not None:
            data["id"] = id
        if key is not None:
            data["key"] = key
        if value is not None:
            data["value"] = value
        if created_at is not None:
            data["createdAt"] = created_at
        if enabled is not None:
            data["enabled"] = enabled

        if self.validation:
            validate_data(data, Secret, SecretAPIError)

        url = urljoin(self.base_url, self.service_path, "secrets")
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise SecretAPIError(response.text, response.status_code)
        return response.json()

    def create_secrets_get(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "secrets:retrieve")
        response = requests.post(url, headers=headers)
        if not response.ok:
            raise SecretAPIError(response.text, response.status_code)
        return response.json()

    def create_secrets_recover(
        self,
        *,
        secret_name: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "secrets/recover/%s" % secret_name
        )
        response = requests.post(url, headers=headers)
        if not response.ok:
            raise SecretAPIError(response.text, response.status_code)
        return response.json()

    def get_secrets_deleted(
        self,
        *,
        secret_name: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "secrets/deleted/%s" % secret_name
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise SecretAPIError(response.text, response.status_code)
        return response.json()

    def get_info(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "info")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise SecretAPIError(response.text, response.status_code)
        return response.json()

    def get_health(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "health")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise SecretAPIError(response.text, response.status_code)
        return response.json()



# generated by datamodel-codegen:
#   filename:  swagger.yaml
#   timestamp: 2024-07-12T07:32:21+00:00

from __future__ import annotations

from datetime import datetime
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field


class UpdateLegalTag(BaseModel):
    name: Optional[str] = Field(
        None, description="The name of the LegalTag", example="OSDU-Private-EHCData"
    )
    contractId: Optional[str] = Field(
        None,
        description="The Id of the physical contract associated with the data being ingested.",
        example="No Contract Related",
    )
    description: Optional[str] = Field(
        None,
        description="The optional description if the LegalTag to allow for easier discoverability of Legaltags overtime.",
    )
    expirationDate: Optional[datetime] = Field(
        None,
        description="The optional expiration date of the contract in the format YYYY-MM-DD",
    )
    extensionProperties: Optional[Dict[str, Dict[str, Any]]] = Field(
        None,
        description="The optional object field to attach any company specific attributes.",
    )


class AppError(BaseModel):
    code: Optional[int] = None
    reason: Optional[str] = None
    message: Optional[str] = None


class Properties(BaseModel):
    countryOfOrigin: Optional[List[str]] = None
    contractId: Optional[str] = None
    expirationDate: Optional[datetime] = None
    originator: Optional[str] = None
    dataType: Optional[str] = None
    securityClassification: Optional[str] = None
    personalData: Optional[str] = None
    exportClassification: Optional[str] = None
    extensionProperties: Optional[Dict[str, Dict[str, Any]]] = None


class RequestLegalTags(BaseModel):
    names: List[str] = Field(
        ...,
        description="The name of all the LegalTags to retrieve.",
        max_items=25,
        min_items=1,
    )


class InvalidTagWithReason(BaseModel):
    name: Optional[str] = Field(None, description="The name of the LegalTag.")
    reason: Optional[str] = Field(
        None, description="The reason the LegalTag is currently invalid."
    )


class InvalidTagsWithReason(BaseModel):
    invalidLegalTags: Optional[List[InvalidTagWithReason]] = Field(
        None, description="A collection of invalid LegalTags"
    )


class SearchLegalTag(BaseModel):
    queryList: Optional[List[str]] = Field(None, description="Filter condition query")
    operatorList: Optional[List[str]] = Field(
        None,
        description="If there are multiple conditions need to be joined in by logical operators",
    )
    sortBy: Optional[str] = None
    sortOrder: Optional[str] = None
    limit: Optional[int] = None


class ReadablePropertyValues(BaseModel):
    countriesOfOrigin: Optional[Dict[str, str]] = Field(
        None,
        description="The values of all the allowed Countries of Origin with the ISO Alpha 2 code and country name.",
    )
    otherRelevantDataCountries: Optional[Dict[str, str]] = Field(
        None,
        description="The values of all the allowed Other Relevant Data Countries with the ISO Alpha 2 code and country name.",
    )
    securityClassifications: Optional[List[str]] = Field(
        None, description="The values of all the allowed Security Classifications."
    )
    exportClassificationControlNumbers: Optional[List[str]] = Field(
        None, description="The name of all the allowed Export Classifications."
    )
    personalDataTypes: Optional[List[str]] = Field(
        None, description="The name of all the allowed Personal Data Type values."
    )
    dataTypes: Optional[List[str]] = Field(
        None, description="The name of all the allowed Data Type values."
    )


class ConnectedOuterService(BaseModel):
    name: Optional[str] = None
    version: Optional[str] = None


class VersionInfo(BaseModel):
    groupId: Optional[str] = None
    artifactId: Optional[str] = None
    version: Optional[str] = None
    buildTime: Optional[str] = None
    branch: Optional[str] = None
    commitId: Optional[str] = None
    commitMessage: Optional[str] = None
    connectedOuterServices: Optional[List[ConnectedOuterService]] = None


class LegalTagDto(BaseModel):
    name: Optional[str] = Field(
        None, description="The name of the LegalTag", example="OSDU-Private-EHCData"
    )
    description: Optional[str] = Field(
        None, description="The description of the LegalTag"
    )
    properties: Optional[Properties] = None


class LegalTagDtos(BaseModel):
    legalTags: Optional[List[LegalTagDto]] = Field(
        None, description="A collection of complete LegalTags"
    )



from __future__ import annotations

import requests

from osdu_client.exceptions import OSDUAPIError
from osdu_client.services.base import OSDUAPIClient
from osdu_client.utils import urljoin
from osdu_client.validation import validate_data

from .models import TranslateItem


class PolicyAPIError(OSDUAPIError):
    pass


class PolicyClient(OSDUAPIClient):
    service_path = ""

    def home_page(
        self,
        *,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent

        url = urljoin(self.base_url, self.service_path, "")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def get_api_policy_v1_policies(
        self,
        *,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        url = urljoin(self.base_url, self.service_path, "api/policy/v1/policies")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def get_fetch_policy(
        self,
        *,
        policy_id: str,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        url = urljoin(
            self.base_url, self.service_path, "api/policy/v1/policies/%s" % policy_id
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def get_fetch_instance_policy(
        self,
        *,
        policy_id: str,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/policy/v1/policies/osdu/instance/%s" % policy_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def get_api_policy_v1_policies_osdu_partition(
        self,
        *,
        policy_id: str,
        data_partition: str,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/policy/v1/policies/osdu/partition/%s/%s" % (policy_id, data_partition),
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def delete_partition_policy(
        self,
        *,
        policy_id: str,
        data_partition: str,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/policy/v1/policies/osdu/partition/%s/%s" % (policy_id, data_partition),
        )
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def create_or_update_partition_policy(
        self,
        *,
        policy_id: str,
        data_partition: str,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/policy/v1/policies/osdu/partition/%s/%s" % (policy_id, data_partition),
        )
        response = requests.put(url, headers=headers)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def evaluate_policy(
        self,
        *,
        policy_id: str,
        include_auth: str | None = None,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        params = {
            "policy_id": policy_id,
        }
        if include_auth is not None:
            params["include_auth"] = include_auth

        url = urljoin(
            self.base_url, self.service_path, "api/policy/v1/evaluations/query"
        )
        response = requests.post(url, headers=headers, params=params)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def translate_policy_api(
        self,
        *,
        query: str,
        input: dict,
        unknowns: list[str],
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        data = {
            "query": query,
            "input": input,
            "unknowns": unknowns,
        }

        if self.validation:
            validate_data(data, TranslateItem, PolicyAPIError)

        url = urljoin(self.base_url, self.service_path, "api/policy/v1/translate")
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def get_api_policy_v1_info(
        self,
        *,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent

        url = urljoin(self.base_url, self.service_path, "api/policy/v1/info")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def create_api_policy_v1_compile(
        self,
        *,
        metrics: str | None = None,
        instrument: str | None = None,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        params = {}
        if metrics is not None:
            params["metrics"] = metrics
        if instrument is not None:
            params["instrument"] = instrument

        url = urljoin(self.base_url, self.service_path, "api/policy/v1/compile")
        response = requests.post(url, headers=headers, params=params)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def get_tenant(
        self,
        *,
        all_data: str | None = None,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        params = {}
        if all_data is not None:
            params["all_data"] = all_data

        url = urljoin(self.base_url, self.service_path, "api/policy/v1/tenant")
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def update_tenant(
        self,
        *,
        service: str,
        polling_min_delay_seconds: str | None = None,
        polling_max_delay_seconds: str | None = None,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        params = {
            "service": service,
        }
        if polling_min_delay_seconds is not None:
            params["polling_min_delay_seconds"] = polling_min_delay_seconds
        if polling_max_delay_seconds is not None:
            params["polling_max_delay_seconds"] = polling_max_delay_seconds

        url = urljoin(self.base_url, self.service_path, "api/policy/v1/tenant")
        response = requests.put(url, headers=headers, params=params)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def delete_tenant(
        self,
        *,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        url = urljoin(self.base_url, self.service_path, "api/policy/v1/tenant")
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def get_health(
        self,
        *,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent

        url = urljoin(self.base_url, self.service_path, "api/policy/v1/health")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def get_ready(
        self,
        *,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent

        url = urljoin(self.base_url, self.service_path, "api/policy/v1/ready")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def validate_policy(
        self,
        *,
        policy_id: str,
        template: str | None = None,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        params = {}
        if template is not None:
            params["template"] = template

        url = urljoin(
            self.base_url, self.service_path, "api/policy/v1/validate/%s" % policy_id
        )
        response = requests.put(url, headers=headers, params=params)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def get_backup(
        self,
        *,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        url = urljoin(self.base_url, self.service_path, "api/policy/v1/backup")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def bootstrap(
        self,
        *,
        force: str | None = None,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        params = {}
        if force is not None:
            params["force"] = force

        url = urljoin(self.base_url, self.service_path, "api/policy/v1/bootstrap")
        response = requests.post(url, headers=headers, params=params)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()

    def get_api_policy_v1_config(
        self,
        *,
        correlation_id: str | None = None,
        user_agent: str | None = None,
        x_user_id: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if correlation_id is not None:
            headers["correlation-id"] = correlation_id
        if user_agent is not None:
            headers["user-agent"] = user_agent
        if x_user_id is not None:
            headers["x-user-id"] = x_user_id

        url = urljoin(self.base_url, self.service_path, "api/policy/v1/config")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PolicyAPIError(response.text, response.status_code)
        return response.json()



# generated by datamodel-codegen:
#   filename:  swagger.yaml
#   timestamp: 2024-07-12T07:32:19+00:00

from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field, constr


class AppError(BaseModel):
    code: Optional[int] = None
    reason: Optional[str] = None
    message: Optional[str] = None


class DdmsInterface(BaseModel):
    entityType: Optional[constr(pattern=r"^[A-Za-z0-9-]{2,50}")] = None
    schema_: Dict[str, Any] = Field(..., alias="schema")


class TestActionResult(BaseModel):
    url: Optional[str] = None
    errors: Optional[str] = None


class CreatedOnEpoch(BaseModel):
    seconds: Optional[int] = None
    nanos: Optional[int] = None


class Acl(BaseModel):
    viewers: Optional[List[str]] = None
    owners: Optional[List[str]] = None


class Data(BaseModel):
    msg: Optional[str] = Field(None, example="Hello")
    uri: Optional[str] = Field(None, example="https://myproj.com/abc123")


class Legal(BaseModel):
    legaltags: Optional[List[str]] = None
    otherRelevantDataCountries: Optional[List[str]] = None


class Filter(BaseModel):
    entityType: Optional[List[constr(pattern=r"^[a-z0-9-]{0,50}")]] = None
    source: Optional[List[constr(pattern=r"^[a-z0-9-]{0,50}")]] = None
    version: Optional[List[constr(pattern=r"^[0-9.]{0,10}")]] = None


class GetSubscriptionResult(BaseModel):
    id: Optional[str] = Field(None, example="dGVzdC1uYW1l")
    name: Optional[str] = Field(None, example="test-subscription")
    description: Optional[str] = Field(None, example="test description")
    topic: Optional[str] = Field(None, example="data-changed-v1")
    pushEndpoint: Optional[str] = Field(None, example="https://myListener.com")
    createdBy: Optional[str] = Field(None, example="test@myapp.com")
    notificationId: Optional[str] = Field(
        None, example="de-6ee609b9-620e-477b-86f9-3b8907643a12"
    )


class SecretType(Enum):
    HMAC = "HMAC"
    GSA = "GSA"


class Secret(BaseModel):
    secretType: Optional[SecretType] = None
    value: Optional[Dict[str, Any]] = Field(None, example="hmacsecret")


class CreatedOnEpochModel(BaseModel):
    seconds: Optional[int] = Field(None, example=1572275081)
    nanos: Optional[int] = Field(None, example=51000000)


class TopicExample(BaseModel):
    id: Optional[str] = Field(None, example="common:test:123")
    kind: Optional[str] = Field(None, example="common:source:type:1.0.0")
    op: Optional[str] = Field(None, example="deleted")


class ConnectedOuterService(BaseModel):
    name: Optional[str] = Field(None, description="Connected outer service name.")
    version: Optional[str] = Field(None, description="Connected outer service version.")


class Ddms(BaseModel):
    id: Optional[constr(pattern=r"^[A-Za-z0-9-]{2,50}")] = None
    name: Optional[constr(pattern=r"^[A-Za-z0-9- ]{2,50}")] = None
    description: Optional[constr(pattern=r"^[A-Za-z0-9. ]{0,255}")] = None
    contactEmail: Optional[str] = None
    interfaces: Optional[List[DdmsInterface]] = None


class CreateAction(BaseModel):
    id: Optional[constr(pattern=r"^[A-Za-z0-9-]{2,50}")] = None
    name: Optional[constr(pattern=r"^[A-Za-z0-9- ]{2,50}")] = None
    description: Optional[constr(pattern=r"^[A-Za-z0-9. ]{0,255}")] = None
    contactEmail: Optional[str] = None
    img: Optional[constr(pattern=r"http[s]*\S+")] = Field(
        None,
        description="Reference link to an image file that can be usd in an UI to represent the action.",
    )
    url: Optional[constr(pattern=r"http[s]*\S+")] = Field(
        None, example="https://myapp.osdu.com/action/{type}/{id}"
    )
    filter: Optional[Filter] = None


class TestAction(BaseModel):
    action: Optional[CreateAction] = None
    testPayload: Optional[Dict[str, Any]] = None


class Action(BaseModel):
    id: Optional[str] = None
    name: Optional[str] = None
    description: Optional[str] = None
    contactEmail: Optional[str] = None
    img: Optional[str] = None
    url: Optional[str] = Field(
        None, example="https://myapp.osdu.com/action/{type}/{id}"
    )
    filter: Optional[Filter] = None
    createdOnEpoch: Optional[CreatedOnEpoch] = None


class Record(BaseModel):
    id: Optional[str] = None
    kind: Optional[str] = None
    acl: Optional[Acl] = None
    legal: Optional[Legal] = None
    data: Optional[Data] = None


class Subscription(BaseModel):
    name: Optional[constr(pattern=r"^[A-Za-z0-9- ]{2,50}")] = Field(
        None, example="test-subscription"
    )
    description: Optional[constr(pattern=r"^[A-Za-z0-9. ]{0,255}")] = Field(
        None, example="test description"
    )
    topic: Optional[str] = Field(None, example="data-changed-v1")
    pushEndpoint: Optional[str] = Field(None, example="https://myListener.com")
    secret: Optional[Secret] = None


class SubscriptionCreateResult(BaseModel):
    id: Optional[str] = Field(None, example="dGVzdC1uYW1l")
    name: Optional[constr(pattern=r"^[A-Za-z0-9- ]{2,50}")] = Field(
        None, example="test-subscription"
    )
    description: Optional[constr(pattern=r"^[A-Za-z0-9. ]{0,255}")] = Field(
        None, example="test description"
    )
    topic: Optional[str] = Field(None, example="data-changed-v1")
    pushEndpoint: Optional[str] = Field(None, example="https://myListener.com")
    createdBy: Optional[str] = Field(None, example="test@myapp.com")
    notificationId: Optional[str] = Field(
        None, example="de-6ee609b9-620e-477b-86f9-3b8907643a12"
    )
    createdOnEpoch: Optional[CreatedOnEpochModel] = None
    secret: Optional[Secret] = None


class Topic(BaseModel):
    name: Optional[str] = Field(None, example="records-changed")
    description: Optional[str] = Field(
        None,
        example="This notification is sent whenever a record is created, updated or deleted.",
    )
    state: Optional[str] = Field(None, example="ACTIVE")
    example: Optional[List[TopicExample]] = None


class VersionInfo(BaseModel):
    groupId: Optional[str] = Field(None, description="Maven artifact group ID.")
    actifactId: Optional[str] = Field(None, description="Maven artifact ID.")
    version: Optional[str] = Field(None, description="Maven artifact version")
    buildTime: Optional[str] = Field(None, description="Maven artifact build time")
    branch: Optional[str] = Field(None, description="Current git branch")
    commitId: Optional[str] = Field(None, description="Latest commit hash")
    commitMessage: Optional[str] = Field(None, description="Latest commit message")
    connectedOuterServices: Optional[List[ConnectedOuterService]] = Field(
        None, description="Connected outer services information"
    )



from __future__ import annotations

import requests

from osdu_client.exceptions import OSDUAPIError
from osdu_client.services.base import OSDUAPIClient
from osdu_client.utils import urljoin
from osdu_client.validation import validate_data

from .models import CatalogRecord, GuessRequest


class WellboreAPIError(OSDUAPIError):
    pass


class WellboreCommonClient(OSDUAPIClient):
    service_path = ""

    def get_about(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "about")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellboreAPIError(response.text, response.status_code)
        return response.json()

    def recognize_family(
        self,
        *,
        label: str,
        description: str | None = None,
        log_unit: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        request_data = {
            "label": label,
        }
        if description is not None:
            request_data["description"] = description
        if log_unit is not None:
            request_data["log_unit"] = log_unit

        if self.validation:
            validate_data(request_data, GuessRequest, WellboreAPIError)

        url = urljoin(self.base_url, self.service_path, "log-recognition/family")
        response = requests.post(url, headers=headers, json=request_data)
        if not response.ok:
            raise WellboreAPIError(response.text, response.status_code)
        return response.json()

    def update_log_recognition_upload_catalog(
        self,
        *,
        acl: dict,
        data: dict,
        legal: dict,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        request_data = {
            "acl": acl,
            "data": data,
            "legal": legal,
        }

        if self.validation:
            validate_data(request_data, CatalogRecord, WellboreAPIError)

        url = urljoin(
            self.base_url, self.service_path, "log-recognition/upload-catalog"
        )
        response = requests.put(url, headers=headers, json=request_data)
        if not response.ok:
            raise WellboreAPIError(response.text, response.status_code)
        return response.json()

    def get_version(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "version")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise WellboreAPIError(response.text, response.status_code)
        return response.json()
